// JUMA-PA100 Amplifier Controller
// Juha Niinikoski, OH2NLT 06.07.2008
 
// Microchip MPLAB + C30 compiler
// MCU dsPIC30F6014A

// Auto F-sense code 08.07.2008
// Boot loader & Temp scaling 09.07.2008
// EEPROM structures & Encoder simulator 17.07.2008
// Power meter fast attack, slow release 04.08.2008
// Improved F-sense code 05.08.2008
// Yaesu 817 band select tests, check_io_board_type() removed 07.08.2008
// TRX2 serial band set protocol 09.08.2008
// 7,3728MHz XTAL, repeat for UP/DN buttons 13.08.2008
// SWR Alarm, Alarm etc, setup displays 13.08.2008
// PA100 - TRX2 serial protocol 14.08.2008
// Yaesu FT817 band voltage table matched with 4M7 input resistance 16.08.2008
// SWR alarm fine tuning 22.08.2008
// Display clean up, Auto/Man logic changed, Alarm beep added 12.11.2008 v0.04
// RS232 band set & query  active only if TRX2 protocol is selected 15.11.2008
// RS232 printouts only in the test mode, do not send garbage to the TRX2 15.11.2008
// Some text changed, default calibration values changed 15.11.2008
// TX capability added to the service mode 15.11.2008 v1.00 
// PA100 temp meter degree sign added to the soft fonts #4 16.11.2008 v1.00
// TRX2 protocol poll rate adjust added 17.11.2008 v1.00
// Alarms force normal display, some text changes 18.11.2008
// UART module data type cast corrections 18.11.2008
// Band +/- end of range tones changed, OPER ==> STBY while TX on corrected 19.11.2008 v1.01
// Band info, ! = out of band, ? = not known, TX/tx display added 22.11.2008	v1.02
// Poll timer changed from calval to defval, TRX2 poll response timeout, some display text changes 30.11.2008 v1.03
// Service mode factory setup function corrected 16.12.2008 v1.04
// Various typographical errors fixed, minor display formatting improvements - 5B4AIY 23/AUG/2011

const char ver[] = {"1.05c"};				// Software version information, displayed at startup
const char date[] = {"25/Oct/2011"};		// Version date

extern void lcd_putch(unsigned char);		// Added - 5B4AIY

#include "juma-pa100.h"						// JUMA-TRX2 hardware specific definitions
#include "lcd-trx2.h"						// LCD specific definitions
#include "DataEEPROM.h"
#include <stdio.h>

_FOSC(CSW_FSCM_OFF & XT_PLL4);				// 7,3728MHz XT, 29,4912MHz clock = 7,3728MHz cycle
//_FOSC(CSW_FSCM_OFF & XT_PLL8);			// 3,68640MHz XT, 29,4912MHz clock = 7,3728MHz cycle
_FWDT(WDT_OFF);                				// Turn off the Watch-Dog Timer.
_FBORPOR(MCLR_EN & PWRT_64 & BORV_20);   	// Enable MCLR, power-on timer, brown out 2V
//_FGS(GEN_PROT);            				// Disable Code Protection, old compiler
 _FBS(CODE_PROT_OFF);
 _FSS(CODE_PROT_OFF);
 _FGS(CODE_PROT_OFF);

// Global variables
int fcy;									// Clock / kHz
int loop_ctr;
// Baud Rate divisors for 29,4912 MHz osc =  7,3728 MHz FCY 
/* Actual baud rate divisors
[0] 1200	383
[1] 2400	191
[2] 4800	95
[3] 9600	47
[4] 19200	23
[5] 38400	11
[6] 57600	7
[7] 115200	3
*/
#define DEFAULT_BAUD_RATE 3		// Default Baud rate = 9600
//const char br_txt[8][8] = {{"1200  "}, {"2400  "}, {"4800  "} ,{"9600  "}, {"19200 "}, {"38400 "}, {"57600 "}, {"115200 "}};
const char br_txt[8][8] = {{"  1200"}, {"  2400"}, {"  4800"} ,{"  9600"}, {" 19200"}, {" 38400"}, {" 57600"}, {"115200"}};						// Right-justified in field - 5B4AIY

// Timing
int seconds;					// Free running 1 second timer

// Display control
#define MAX_LCD_MODE 1			// Last display "main" page
#define	MAX_SUB_PAGE0 4			// Sub page 0 displays
#define	MAX_SUB_PAGE1 10			// Sub page 1 displays
#define S_BAR 0x07				// S-meter vertical bar character

unsigned char lcdpbuff[20];		// LCD print buffer
int	lcd_mode;					// LCD display mode
int sub_page0;					// LCD display sub page for lcd_mode0
int sub_page1;					// LCD display sub page for lcd_mode1
int service_mode;				// Service mode selected flag

// ADC & meter variables
int batt;						// Power supply voltage scaled value, measured only when displayed
long batt_mean_val;
int bat_samples = 0;
//int s_meter;					// Scaled S-meter value 0 ... 48
int fwd_adval;					// Forward & Reverse power raw values, measured only when TX
int rev_adval;
int id_cur;						// PA Drain current raw value
long pa_temp;					// PA temperature, ADC value
int scaled_pa_temp;				// Scaled to deg C
int batt_pre_limit;				// Pre-limit warning threshold value

int sample_counter;				// Power measurement sample counter
long int fwd_avg;
long int rev_avg;

long fwd_pwr;					// Relative forward power
long rev_pwr;					// Relative reverse power
long out_pwr;					// Scaled output power = fwd_pwr - rev_pwr * scale
long peak_pwr;					// Peak, fast attack, slow decay power
int swr;						// Calculated SWR * 100
int old_swr;					// For alarm trigger
int pwr_meter;					// Scaled S-meter value 0 ... 48

// PA status
int key;						// TX request status
//int pa_state;					// Amplifier state, 0 = Standby, 1 = Operate
//int auto_band;				// Band select type, 0 = Manual, 1 = Auto
unsigned int alarms;			// Alarm bits
int last_man_band;				// Last selected manual band
static int fan_speed;			// Fan speed setting.
static int fan_stop;			// Fan cut-off temperature.

// Band selector display texts
const char bs_txt[11][3] = {{"?! "}, {"1.8"}, {"3.5"}, {"  7"}, {" 10"}, {" 14"}, {" 18"}, {" 21"}, {" 24"}, {" 28"}, {"  ?"}};
const char mb_txt[11][3] = {{"?! "}, {"160"}, {" 80"}, {" 40"}, {" 30"}, {" 20"}, {" 17"}, {" 15"}, {" 12"}, {" 10"}, {" ? "}};

// Band select limits
const unsigned int band_limits[10] = {LM1, LM2, LM4, LM7, LM10, LM14, LM18, LM21, LM24, LM28};

// Yaesu 817 band select ADC #
// This table is with 100k || input, 817 internal impedance is about 6,5k
// Band when over this value                   1.8  3.5  7    10   14    18    21    24    28    50    144   430 MHz
//const unsigned int y817_band_limits[13] = {10, 122, 388, 655, 905, 1155, 1423, 1690, 1931, 2172, 2441, 2709, 2962}; 

// This table is with no load (4M7 || input)
const unsigned int y817_band_limits[13] = {10, 135, 409, 683, 954, 1228, 1503, 1773, 2047, 2322, 2592, 2866, 3141};

unsigned int fd_counter;			// Counts how many times factory defaults have been reloaded
int old[4];							// Frequency Sense variables
int	fsense_tst;						// Flag for F-sense test printouts

// PA config
int fsense_present;					// F-sense board installed

int test_band;						// Test

#include "pa100_eeprom.h"			// EEPROM storage structure

union
	{
	struct defval defval;
	unsigned int storage[sizeof(struct defval) / 2];
	}eeprom;

// System calibration values

union
	{
	struct calval calval;
	unsigned int ee[sizeof(struct calval) / 2];
	}cal;

// External functions

extern void us_delay( unsigned int);				// Delay routines
extern void ms_delay( unsigned int);

extern void init_timers_pwm(void);					// Timers & PWM DAC

extern void set_pwm3_dac(unsigned int);				// LCD back light
extern void set_pwm4_dac(unsigned int);				// LCD contrast

extern void tone_on(int);							// Start tone generator, period length
extern void tone_off(void);							// Stop tone generator
extern void beep(int, int);							// Generate beep, period, length

extern int encoder_get(void);						// Get simulated encoder value

extern void init_adc12(void);
extern int convert_adc12(unsigned int adchs);		// ADC

extern void draw_s_meter(int);						// LCD functions
extern void lcdoutch(unsigned char);
extern void lcd_putst(register const char *);
extern void set_chgen(void);
extern void initlcd(void);
extern void clear_lcd(void);
extern void set_cur_lcd(unsigned char);
extern void disp_meter(double, char);				// New version - 5B4AIY
extern void lcd_putchhex(unsigned char);

extern void InitUSART1(void);						// Terminal functions
extern void SetUSART1baud(int);						// Baud rate select
extern void putch(char);
extern unsigned char kbhit(void);
extern unsigned char getch(void);
extern unsigned char getche(void);
extern unsigned long getlong(void);
extern void bin2ascii(unsigned long);

extern void init_spi1(void);						// SPI, main board 74HC595 I/O latch

extern void service(void);							// Service functions
extern void serial_test(void);						// Serial I/O
extern void dump_eeprom(void);						// Dump EEPROM contents

extern void send_band_data(void);					// Send band data, PA100 protocol
extern void query_band_data(void);					// Query band data, PA100 protocol
extern void serial_pa100(void);						// PA100 protocol engine
extern void clear_buffer(void);						// Clear protocol buffer

// Frequency counter test
//extern unsigned int get_fcounter(void);				// Read last frequency counter
extern unsigned int freq;							// Frequency counter
//extern unsigned int max_freq;
//extern unsigned int max_freq_delay;
//extern unsigned int max_freq_set;

extern int band_bins[10];							// Found samples
extern int band_meas_ready;							// Band select measurement ready
extern int freq_sample_ctr;							// Sample counter

// External variables
extern unsigned int long_push;						// Long push timer, count from set value to zero, global visibility
extern int blink;									// Alarm blink flag
extern int decay_counter;							// Power meter slow decay
extern unsigned int band_query_timer;				// Band query timer
extern int poll_resp_rec;							// Poll response (freq set) received from TRX2, 1 = received
extern unsigned int t_stamp;						// 1ms free running time stamp

// Helper functions

// Set factory defaults for EEPROM storage

void set_factory_defaults(void)
	{
	int x;
// User settings
	for(x = 0; x < 11; x++)
		{
		eeprom.defval.rf_gain[x] = 0;				// Minimum gain
		}

	eeprom.defval.band = MAX_BAND;					// 28MHz default band
	eeprom.defval.contrast = DEFAULT_CONTRAST;		// 2000
	eeprom.defval.back_light = DEFAULT_BL;			// 300
	eeprom.defval.serial_mode = SERIAL_MODE;		// Juma TRX-2 (PA-100 Protocol)
	eeprom.defval.br = DEFAULT_BAUD_RATE;			// Initial Baud rate = 9600
	eeprom.defval.bsel_mode = 0;					// F-sense in use
	eeprom.defval.swr_limit = SWR_LIMIT;			// SWR alarm trip point
	eeprom.defval.Temp_Units = 1;					// Temperature units, 1 = C, 0 = F
	eeprom.defval.Fan_Control = 0;					// Fan Speed Control 0 = Normal, 1 = Low, 2 = Medium, 3 = High
	eeprom.defval.temp_limit = TEMP_LIMITC;			// Temperature alarm
	eeprom.defval.fan_start = FAN_STARTC;			// FAN start point
	eeprom.defval.pa_state = 0;						// Amplifier state, 0 = Standby, 1 = Operate
	eeprom.defval.auto_band = 0;					// Band select type, 0 = Manual, 1 = Auto
	eeprom.defval.Band_Units = 0;					// 0 = MHz, 1 = Metres

// Calibration values
	cal.calval.id_mult = ID_MULT;					// I = (ADC * ID_MULT) / 200000 
	cal.calval.batt_mult = BATT_MULT;				// V = (ADC * BATT_MULT) / 1000000
//	cal.calval.s_mtr_mult = S_MTR_MULT;				// Not used
	cal.calval.rev_pwr_mult = REV_PWR_MULT;			// 
	cal.calval.fwd_pwr_mult = FWD_PWR_MULT;			// P = (((ADC * ADC) / 10) * FWD_PWR_MULT) / 10000000
	cal.calval.beep_len = DEFAULT_BEEP_LEN;			// 50mS
	cal.calval.q_timer = POLL_TIMER;				// 8 seconds Band query timer (ms)
	cal.calval.samples = 1;							// Samples count for power measurements and SWR 
	cal.calval.alarm_flags = 0x03;					// Under & Over-voltage alarms on
	cal.calval.overvoltage_trip = OVERVOLTAGE_TRIP;	// Nominally 14.5V
	cal.calval.undervoltage_trip = UNDERVOLTAGE_TRIP;// Nominally 11.0V

// Increment factory default reset counter
	ReadEE(EEPAGE, EE_FD_LOC, &fd_counter, WORD);
	fd_counter++;

// Save factory defaults reset counter
	EraseEE(EEPAGE, EE_FD_LOC, WORD);
	WriteEE(&fd_counter, EEPAGE, EE_FD_LOC, WORD);
	}

// Save defaults

void save_defaults(void)
	{
	int x, y;

	eeprom.defval.d_csum = 0;
	for(x = 0; x < (sizeof(struct defval) / 2); x++)
		{
		if(x < ((sizeof(struct defval) / 2) - 1))
			eeprom.defval.d_csum = eeprom.defval.d_csum + eeprom.storage[x]; 	// Calculate mod 0xFFFF checksum
		else
			eeprom.defval.d_csum = 0 - eeprom.defval.d_csum;					// Last round, invert checksum

 		y = EraseEE(EEPAGE, (2 * x + EEDEF), WORD);
		y = WriteEE(&(eeprom.storage[x]), EEPAGE, (2 * x + EEDEF), WORD);		// EEPROM Address 8 high bits, address + physical EEPROM start 16 low bits
		}

 	if(eeprom.defval.serial_mode == 1)											// Conditional print only if RS232 = Test
		printf("\n\rWords = %d, Success = %d, Checksum = %X", x, y, eeprom.defval.d_csum);	// Print EEPROM checksum
	}

// Read defaults

unsigned int read_defaults(void)
	{
	int x, y;
	unsigned int csum;

	csum = 0;

	for(x = 0; x < (sizeof(struct defval) / 2); x++)
		{
		y = ReadEE(EEPAGE, (2 * x + EEDEF), &(eeprom.storage[x]), WORD);	// EEPROM Address 8 high bits, address + physical EEPROM start 16 low bits
// Add error checking ?  y = success code

		csum += eeprom.storage[x];								
		}

// Read factory defaults reset counter
	ReadEE(EEPAGE, EE_FD_LOC, &fd_counter, WORD);

	return csum;
	}

// Save calibration values

void save_calval(void)
	{
	int x, y;

	cal.calval.c_csum = 0;
	for(x = 0; x < (sizeof(struct calval) / 2); x++)
		{
		if(x < ((sizeof(struct calval) / 2) - 1))
			cal.calval.c_csum = cal.calval.c_csum + cal.ee[x];	// Calculate mod 0xFFFF checksum
		else
			cal.calval.c_csum = 0 - cal.calval.c_csum;

		y = EraseEE(EEPAGE, (2 * x + EECAL), WORD);
		y = WriteEE(&(cal.ee[x]), EEPAGE, (2 * x + EECAL), WORD);	// EEPROM Address 8 high bits, address + physical EEPROM start 16 low bits
		}

 	if(eeprom.defval.serial_mode == 1)							// Conditional print only if RS-232 = Test
		printf("\n\rWords = %d, Success = %d, Checksum = %X", x, y, cal.calval.c_csum);
	}

// Read calibration values

unsigned int read_calval(void)
	{
	int x, y;
	unsigned int csum;

	csum = 0;
	for(x = 0; x < (sizeof(struct calval) / 2); x++)
		{
		y = ReadEE(EEPAGE, (2 * x + EECAL), &(cal.ee[x]), WORD);	// EEPROM Address 8 high bits, address + physical EEPROM start 16 low bits
		csum = csum + cal.ee[x];
		}
	return csum;
	}

// Calculate SWR

int calc_swr(int f, int r)
	{
	long Ro;
	
	if(f < 50) return 0;						// Only calculate if we have some power
	
	Ro = ((long)r * 100L) / (long)f;
	return (int)(((100 + Ro) * 100) / (100 - Ro));
	}

// Display Alarms

void display_alarms(void)
	{
	if(loop_ctr == (LOOP_COUNT / 2) || loop_ctr == 0)
		beep(1843, 100);

	if(loop_ctr == 0)
		loop_ctr = LOOP_COUNT;

	if(loop_ctr > (LOOP_COUNT / 2))
		{
		if(alarms & SWR_AL)
			lcd_putst(" SWR ");	
		else
			{
			if(alarms & CURR_AL)
				lcd_putst(" CURR");
			else
				{
				if(alarms & TEMP_AL)
					lcd_putst(" TEMP");
				else
					{
					if(alarms & HI_V)
						lcd_putst(" Hi-V");
					else
						{
						if(alarms & LO_BATT)
							lcd_putst(" Batt");
						else
							{
							if(alarms & LO_V)
								lcd_putst(" Lo-V");
							}
						}
					}
				}
			}
		}
	else
		lcd_putst("     ");						// Show nothing

	loop_ctr--;
	}

// Calculate & display bar graph power (forward voltage) meter

void display_pwr_meter(void)
	{
	pwr_meter = (fwd_adval * F_VOLT_GRAPH_MULT) >> 17; 	// Forward voltage display
	draw_s_meter(pwr_meter);							// Full bar = 160W
	}

void disp_fwd_pwr(void)
	{
	double temp;
	long decay_term;

	if(key != 0)								// TX on ?
		{
		if(out_pwr > peak_pwr)					// Fast attack
			{
			peak_pwr = out_pwr;					// Fast attack
			decay_counter = PEAK_SHOW_TIME;		// Reload decay timer
			}
		else									// Slow release
			{
			if(decay_counter == 0)
				{
				decay_counter = DECAY_TIME;		// Reload decay timer
				decay_term = peak_pwr >> DECAY_RATE;

				if(decay_term == 0)				// Zero reached
					peak_pwr = 0;
				else
					peak_pwr = peak_pwr - decay_term;	// Decay
				}
			}
		temp = peak_pwr;
		temp *= cal.calval.fwd_pwr_mult;
		temp /= 100000000.0;
		disp_meter(temp, 'W');					// Scaling, P = (((ADC * ADC) / 10) * Calibration Factor) / 10,000,000
		}
	else
		{
		lcd_putst("PWR---W");
		peak_pwr = 0;
		}
	}

// Measure & display Drain current meter

void disp_id(void)
	{
	if(key)												// TX on ?
		{
		id_cur = ((long)convert_adc12(ID_CUR) * cal.calval.id_mult) / 200;
		disp_meter((double)id_cur, 'A');				// I = id_cur / 10000
		}
	else
		lcd_putst("I --.-A");
	}

// Calculate & measure PA temperature

void disp_temp(void)
	{
	if(eeprom.defval.Temp_Units)
		disp_meter((double)scaled_pa_temp, 'C');
	else
		disp_meter((double)scaled_pa_temp, 'F');
	}

// Check if any alarms

void check_alarms(void)
	{
	int v;

	v = convert_adc12(BATT_CH);
	
	if((cal.calval.alarm_flags & 0x01) && (v > cal.calval.overvoltage_trip))
		alarms = alarms | HI_V;

	if((v < batt_pre_limit) && !(alarms & LO_BATT))		// If the low-voltage alarm is disabled, then batt_pre_limit is set to 0 as part of the start-up sequence.
		alarms = alarms | LO_BATT;

	if((cal.calval.alarm_flags & 0x02) && (v < cal.calval.undervoltage_trip))
		alarms = alarms | LO_V;

// Check over current alarm bit & set alarm
//	if(OC && !(alarms & CURR_AL))
	if(OC)
		{
		TX_ON = 0;										// RF off immediately
		alarms = alarms | CURR_AL;						// Set alarm flag
		}

// Check PA temperature
	if(scaled_pa_temp > eeprom.defval.temp_limit)
		{
		TX_ON = 0;										// RF off immediately
		alarms = alarms | TEMP_AL;						// Set alarm flag
		}

// Check SWR, two samples needed for trip
// Note, SWR alarm is active also in STBY
	if((swr > eeprom.defval.swr_limit) && (old_swr > eeprom.defval.swr_limit))
		{
		TX_ON = 0;										// RF off immediately
		alarms = alarms | SWR_AL;						// Set alarm flag
		}

	if(alarms)
		lcd_mode = 0;									// Force normal display
	}

// Clear alarms & try to reset over-current latch

void clear_alarms(void)
	{
	loop_ctr = 0;

	if(!alarms)
		return;

	if(alarms & SWR_AL)
		{
		alarms = alarms & 0x7D;
		return;
		}

	if(alarms & CURR_AL)
		{
		alarms = alarms & 0x7B;					// Clear alarm flag
		OC_CLR = 0;								// Generate reset pulse for over-current latch
		ms_delay(20);
		OC_CLR = 1;
		return;
		}

	if(alarms & TEMP_AL)
		{
		alarms = alarms & 0x77;
		return;
		}

	if(alarms & HI_V)
		{
		alarms = alarms & 0x6E;
		return;
		}

	if(alarms & LO_BATT)
		{
		batt_pre_limit = 0;
		alarms = alarms & 0x5E;
		return;
		}

	if(alarms & LO_V)
		{
		alarms = alarms & 0x3E;
		return;
		}
	return;
	}

// Handle PWR button.
void power_off(void)
	 {
	beep(HZ466_85, cal.calval.beep_len);								// Notify PWR button pressed

	if(alarms)
		{
		TX_ON = 0;														// RF off immediately
		eeprom.defval.pa_state = 0;
		long_push = MEDIUM_PUSH;										// Prepare for emergency shutdown
		clear_alarms();
		
		while(PWR_SW)
			{
			if(long_push == 0)
				{
				clear_lcd();
				lcd_putst("   Emergency    ");
				set_cur_lcd(LINE2);
				lcd_putst("    Shutdown    ");

				while(PWR_SW);

				PWR_ON = 0;
				for(;;);
				}
			}
		ms_delay(BUTTON_DEBOUNCE);
		return;
		}

	clear_lcd();
	long_push = POWER_OFF;												// Load power-off guard timer

	while(PWR_SW)
		{
		set_cur_lcd(LINE1);
		sprintf(lcdpbuff, "Power down in %2i", (long_push / 300));		// Display off counter, 5,4,3,2,1,0
		lcd_putst(lcdpbuff);
		set_cur_lcd(LINE2);
		lcd_putst("                ");

		if(long_push == 0)
			{
			clear_lcd();
			beep(HZ392_01, cal.calval.beep_len * 2);					// Play low sound when ready for power down
			save_defaults();

			if(eeprom.defval.serial_mode == 1)							// Conditional print only if RS-232 = Test
				printf("\n\rUser settings saved to EEPROM, Power OFF\n\r");

			ms_delay(5);												// Let the printout finish
			set_pwm3_dac(0);											// Turn backlighting off
			FAN1 = 0;
			FAN2 = 0;													// Turn fan off

			while(PWR_SW);												// Wait for power switch to be released,

			PWR_ON = 0;													// Unlock the power latch
			for(;;);													// and walk off into that long goodnight...
			}	
		}
	return;
	}
 
// RS-232 loop back test
void rs232_test(void)
	{
	unsigned char c;

	clear_lcd();
	lcd_putst("RS-232 Test     ");			// Changed prompt - 5B4AIY
	set_cur_lcd(LINE2);
	lcd_putst("Awaiting data...");			// Changed prompt - 5B4AIY

	for(;;)
		{
		if(kbhit())
			{
			c = getch();
			beep(HZ698_45, 10);				// Give beep tone
			putch(c);						// Echo character
			set_cur_lcd(LINE2);
			lcd_putst("RX Data: ");
			lcd_putch(c);					// Display "as received"
			lcd_putst("  0x");
			lcd_putchhex(c);				// Display in HEX format
			}

		if(PWR_SW == 1)						// Exit test
			break;
		}
	}

// F-Sense auto band

void reset_fsense(void)						// Clear F-sense 
	{
	int x;

	for(x = 0; x < 10; x++)
		{
		band_bins[x] = 0;
		}

	freq_sample_ctr = F_SAMPLES;			// Reload sample counter
	band_meas_ready = 0;					// Clear measurement ready flag, start new round
	}

void eval_band(void)						// Improved F-sense
	{
	int x;

	if(band_meas_ready != 0)				// Decide when we have complete set of measurements available
		{
		if(fsense_tst != 0)
			{
			for(x = 0; x < 10; x++)			// Test printout
				{
				printf("%i ", band_bins[x] );
				}
			printf("\n\r");
			}

		if(band_bins[0] < V_SAMPLES)		// We have meaningful sample set, enough measurements != 0
			{
			for(x = 9; x > 0; x--)			// Find highest measured frequency from this sample set
				{
				if(band_bins[x] != 0)
					break;
				}
			eeprom.defval.band = x;			// Set band
			}

		reset_fsense();						// Start next round
		}
	}

// Check I/O board type
void check_io_board_type(void)
	{
	int x;

	x = convert_adc12(B_TYPE);				// Get board ID voltage
	fsense_present = 1;						// F-sense board always present
	}

// Check Yaesu 817 band data (voltage)
int eval_y817_band(void)
	{
	int x, bvolt;
	bvolt = convert_adc12(Y_817);			// Get Yaesu 817 band data voltage

	for(x = 10; x > 0; x--)
		{
		if(bvolt > y817_band_limits[x])
			break;
		}
	return x;
	}

// Set gain & filter relays
void set_relays(void)
	{
// Set RF gain relays
	switch(eeprom.defval.rf_gain[eeprom.defval.band])
		{	
		case 0:					// Minimum gain, (Max attenuation)
			DB_2 = 1;
			DB_4 = 1;
		break;

		case 1:
			DB_2 = 0;
			DB_4 = 1;
		break;

		case 2:
			DB_2 = 1;
			DB_4 = 0;
		break;

		case 3:					// Maximum gain, (no attenuation)
			DB_2 = 0;
			DB_4 = 0;
		break;
		}

// Set RF Filter relays, brute force method 
	switch(eeprom.defval.band)
		{
		 case 1:				// 1.8 MHz
			M1_8   = 1;
			M3_5   = 0;
			M7     = 0;
			M10    = 0;
			M14_18 = 0;
			M21_28 = 0;
		break;

		case 2:					// 3.5 MHz
			M1_8   = 0;
			M3_5   = 1;
			M7     = 0;
			M10    = 0;
			M14_18 = 0;
			M21_28 = 0;
		break;

		case 3:					// 7 MHz
			M1_8   = 0;
			M3_5   = 0;
			M7     = 1;
			M10    = 0;
			M14_18 = 0;
			M21_28 = 0;
		break;

		case 4:					// 10 MHz
			M1_8   = 0;
			M3_5   = 0;
			M7     = 0;
			M10    = 1;
			M14_18 = 0;
			M21_28 = 0;
		break;

		case 5: case 6:			// 14 & 18 MHz
			M1_8   = 0;
			M3_5   = 0;
			M7     = 0;
			M10    = 0;
			M14_18 = 1;
			M21_28 = 0;
		break;

		case 7: case 8:	case 9:	case 10: case 0: default:	// 21, 24, 28 MHz, 10 = not known, 0 = out of band
			M1_8   = 0;
			M3_5   = 0;
			M7     = 0;
			M10    = 0;
			M14_18 = 0;
			M21_28 = 1;
		break;
		}	
	}

// Auto band select
void auto_band_select(void)
	{
	if(eeprom.defval.auto_band != 0)
		{
		switch(eeprom.defval.bsel_mode)
			{
			case 0:
				eval_band();											// Set band selector with F-sense
			break;

			case 1:
				if(band_query_timer == 0 && cal.calval.q_timer != 0)	// Check if time to ask band data
					{
					band_query_timer = cal.calval.q_timer;

					if(eeprom.defval.bsel_mode == 1)					// Transmit query only when TRX2 protocol selected
						{
						if(poll_resp_rec == 0)							// We have not received set band since last query
							eeprom.defval.band = NOT_KNOWN;				// Band information lost
						query_band_data();
						poll_resp_rec = 0;								// Reset poll response flag	
						}
					}
			break;

			case 2:
				eeprom.defval.band = eval_y817_band();					// Set band selector with FT817 band voltage
			break;
			}
		}
	}

// Do analog measurements
void analog_measurements(void)
	{
// Measure temperature
	pa_temp = convert_adc12(TEMP);
			
	if(eeprom.defval.Temp_Units)
		scaled_pa_temp = (pa_temp * T_MULTC) >> 16;						// Scale temperature
	else
		scaled_pa_temp = ((pa_temp * T_MULTF) >> 16) + 32;				// Scale temperature
// Convert fwd & rev A/D channels

	if(sample_counter)
		{
		fwd_avg += (long)convert_adc12(FWD_PWR);
		rev_avg += (long)convert_adc12(REV_PWR);
		sample_counter--;
		}
	else
		{
		fwd_avg /= (long)cal.calval.samples;
		rev_avg /= (long)cal.calval.samples;
		fwd_adval = (int)fwd_avg;
		rev_adval = (int)rev_avg;
		fwd_avg = rev_avg = 0;
		sample_counter = cal.calval.samples;
		}

// Store non-scaled values for later use
		fwd_pwr = (long)fwd_adval * (long)fwd_adval;	// Calculate relative power levels ( U^2 )
		rev_pwr = (long)rev_adval * (long)rev_adval;
		out_pwr = fwd_pwr - rev_pwr;					// Output power = forward - reverse.
		old_swr = swr;									// Shift sample queue
		swr = calc_swr(fwd_adval, rev_adval);			// Calculate SWR
	}

// PA cooling fan control
// This slightly more complex method avoids unnecessary switching of the fan as we progress
// through the various choices. If we simply turned the various control bits on as we climbed
// through the temperature range, on each pass through the main loop we would sequentially switch
// the various fan control bits which might generate a transient. This method ensures that we select
// the desired fan speed immediately, and only switch when necessary. The Fan Control logic now has
// an additional parameter. The user can select whether the fan is on all the time, and the speed
// at which it will run. In the User Configuration menu, the Fan Control option can be set to Normal,
// in which case the fan speed will be controlled entirely by the heat-sink temperature, or Low,
// Medium, or High, in which case the fan will run at the selected speed all the time.
// In order to avoid "fan-twitch", the fan speed control is a global variable that retains its value.
// Fan-twitch occurs because as the teperature reduces, there was no hysteresis, and thus the fan would
// be commanded on and off repeatedly. The fan_stop parameter is calculated once on start-up, or whenever
// the temperature scale is changed, or the start temperature is changed.

void fan_control(void)
	{
	if(scaled_pa_temp <= fan_stop)
		fan_speed = 0;			// Fan OFF

	if(scaled_pa_temp >= eeprom.defval.fan_start)
		fan_speed = 1;			// Fan Low Speed
	
	if(eeprom.defval.Temp_Units)										// Celsius Units
		{
		if(scaled_pa_temp >= (eeprom.defval.fan_start + F_speed_2C))	// Start Speed + 5C
			fan_speed = 2;		// Fan Medium Speed

		if(scaled_pa_temp >= (eeprom.defval.fan_start + F_speed_3C))	// Start Speed + 10C
			fan_speed = 3;		// Fan High Speed
		}
	else																// Fahrenheit Units
		{
		if(scaled_pa_temp >= (eeprom.defval.fan_start + F_speed_2F))	// Start Speed + 10F
			fan_speed = 2;

		if(scaled_pa_temp >= (eeprom.defval.fan_start + F_speed_3F))	// Start Speed + 20F
			fan_speed = 3;
		}

	fan_speed += eeprom.defval.Fan_Control;								// Select whether fan runs all the time
	
	if(fan_speed > 3)
		fan_speed = 3;

	switch(fan_speed)
		{
		case 0:
			FAN1 = 0;			// High speed, Fan Off
			FAN2 = 0;			// Low speed
		break;

		case 1:
			FAN1 = 0;			// High speed, Low fan speed
			FAN2 = 1;			// Low speed
		break;

		case 2:
			FAN1 = 1;			// High speed, Medium fan speed
			FAN2 = 0;			// Low speed
		break;

		case 3:
			FAN1 = 1;			// High speed, Maximum fan speed
			FAN2 = 1;			// Low speed
		break;
		}
	}

// main __________________________________________________________________________________

int main(void)
	{
	int temp, temp_units;			// Added - 5B4AIY Used to determine whether to reset the start temperature
	unsigned int y, w;
	unsigned long ultemp;
	int loop_counter;

// Set general I/O
	TRISA = INIT_TRISA;				// SWITCHES, PTT
	PORTA = INIT_PORTA;

	TRISB = INIT_TRISB;				// ADC inputs, TXEN, RXEN, General I/O
	PORTB = INIT_PORTB;

	TRISC = INIT_TRISC;				// Sideband select & switches
	PORTC = INIT_PORTC;

	TRISD = INIT_TRISD;				// LEDs & LCD pins = output
	PORTD = INIT_PORTD;

	TRISF = INIT_TRISF;				// DDS controls
	PORTF = INIT_PORTF;

	TRISG = INIT_TRISG;				// DDS controls, EEPROM, DScard signals
	PORTG = INIT_PORTG;

// Test fan
	FAN2 = 1;						// Test FAN2, low speed on

// Power switch
	PWR_ON = 1;						// Latch power ON

// Start PWM system
	IPC0 = 0X5444;					// Set tone generator (TMR1) priority higher than others
	init_timers_pwm();				// Setup PWM & tone generators

	fcy = FCY / 1000;				// Clock freq (kHz)

// Init Main Board SPI traffic
	init_spi1();

// Start LCD
	initlcd();						// Initialise LCD
	set_chgen();					// Load bar graph fonts

// Put hello messages to LCD
	clear_lcd();
	lcd_putst("JUMA-PA100 ");		// Say hello
	lcd_putst(ver);					// Display version
	set_cur_lcd(LINE2);
	lcd_putst("  OH2NLT OH7SV  ");	// Say who we are

// Start UART
	InitUSART1();					// Wake up serial interface

// Start ADC
	init_adc12();					// Setup A/D converter, must be done before Keyer is started, RB I/O bits

// Check if service mode start
	service_mode = 0;
	long_push = VERY_LONG_PUSH;		// Set long push timer

	while(PWR_SW == 1)
		{
		if(long_push == 0)
			{
			beep(HZ392_01, 300);	// Give service beep
			set_cur_lcd(LINE2);	
			lcd_putst("  Service Mode  ");
			service_mode = 1;
	
			while(PWR_SW == 1);		// Show message as long as Power switch is pressed

			set_cur_lcd(LINE1);
			lcd_putst("Push DISPLAY nxt");	// Display some instructions
			set_cur_lcd(LINE2);
			lcd_putst("Push OPER = Save");
			}
		}

// Read calibration & default values from EEPROM
	y = read_calval();
	w = read_defaults();

 	if(eeprom.defval.serial_mode == 1)	// Conditional print only if RS-232 = Test
		printf("\n\rEEPROM checksums, Cal = %X, Def Cs = %X, Factory default resets = %d",y, w, fd_counter);

	if(y != 0 || w != 0)			// Compare checksums
		{
		dump_eeprom();				// Test, show EEPROM content before reset
		printf("\n\rEEPROM checksum error, reload factory defaults");
		clear_lcd();
		set_cur_lcd(LINE1);
		lcd_putst(" Checksum Error ");
		set_cur_lcd(LINE2);
		lcd_putst("Loading Defaults");
		ms_delay(1500);
		set_factory_defaults();		// EEPROM empty or corrupted, set factory defaults
		save_calval();
		save_defaults();
		}

// Do calibration functions if service mode start
	if(service_mode)
		service();

// Check if factory defaults asked
	while(!BAND_UP)								// If Band+ button pressed ask user what to do
		{
		clear_lcd();
		set_cur_lcd(LINE1);
		lcd_putst("Reset Defaults?");
		set_cur_lcd(LINE2);
		lcd_putst("BAND+:Y BAND-:N");

		while(!BAND_UP);						// Wait for button to be released

		ms_delay(BUTTON_DEBOUNCE);
		
		while(BAND_UP && BAND_DN);				// Wait for a button push

		set_cur_lcd(LINE2);
		
		if(!BAND_UP)
			{
			beep(HZ698_45, 50);
			set_factory_defaults();				// EEPROM empty, set factory defaults
			save_calval();
			save_defaults();
			lcd_putst("Restored        ");
			}
		else
			{
			beep(HZ587_31, 50);
			lcd_putst("Aborted         ");
			}
		ms_delay(1000);
		}

// Set LCD contrast & back light with EEPROM data
	set_pwm4_dac(eeprom.defval.contrast);
	set_pwm3_dac(eeprom.defval.back_light);

// Set display mode etc
	lcd_mode = 0;									// Select default display
	sub_page0 = 0;

// General settings
	if(eeprom.defval.band > MAX_BAND)				// If last shut down was with undefined frequency
		eeprom.defval.band = MAX_BAND;

// Start-up beep
	beep(HZ698_45, cal.calval.beep_len * 2);		// Give start-up beep
	OC_CLR = 0;										// Clear over-current latch
	FAN1 = 1;										// High speed on, fan test phase 2
	FAN2 = 0;										// Test FAN2, low speed off
	ms_delay(2000);									// Show start-up message
	clear_lcd();
	OC_CLR = 1;
	FAN1 = 0;										// Fan test end
	FAN2 = 0;

// Check I/O boards
// Note! Present hardware does not support board type, Btype signal is used for Yaesu band data
	check_io_board_type();
//	if(fsense_present != 0)
//			printf("\n\rF-Sense board present, Auto Band feature operational");

// Set user baud rate
	SetUSART1baud(eeprom.defval.br);

// Conditional start up text printout, print only if RS-232 = Test
 	if(eeprom.defval.serial_mode == 1)
		{
// hello message
		printf("\n\r\n\rJUMA-PA100 / ");
		printf(ver);
		printf(" / ");
		printf(date);
		printf("\n\rCopyright Juha Niinikoski, OH2NLT\n\r\n\r");
		printf("System Clock = %i kHz\n\r", fcy);

// Serial Baud rate info
		if(eeprom.defval.br != DEFAULT_BAUD_RATE)
			printf("\n\rSet UART user baud rate to #%i,  %sbd\n\r", eeprom.defval.br, br_txt[eeprom.defval.br]);	// print with user baud rate
		}

// RS-232 I/O Test
	if(DISP == 0)						// If Config/DISPLAY pressed during startup
		rs232_test();					// Goto RS-232 test loop

// Test, Freq CTR display
 	sub_page1 = 0;
 	lcd_mode = 0;

// Other
 	last_man_band = NOT_KNOWN;			// Auto to manual old manual band select value
	temp_units = eeprom.defval.Temp_Units;
	eeprom.defval.pa_state = 0;			// Always start in standby state
	sample_counter = cal.calval.samples;
	fwd_avg = 0;
	rev_avg = 0;
	fwd_adval = 0;
	rev_adval = 0;
	loop_counter = 3125;
	loop_ctr = 0;

	if(cal.calval.alarm_flags & 0x02)
		{
		ultemp = ((long)(cal.calval.undervoltage_trip) * (long)(cal.calval.batt_mult)) + 100000UL;
		batt_pre_limit = (int)(ultemp / (long)cal.calval.batt_mult);
		}
	else
		batt_pre_limit = 0;

	if(eeprom.defval.Temp_Units)
		fan_stop = eeprom.defval.fan_start - 2;
	else
		fan_stop = eeprom.defval.fan_start - 2;

	fan_speed = 0;						// Initialise the fan speed.

// Main forever loop _____________________________________________________________________________________________________________

	for(;;)
		{

// Test counter to determine loop cycle time.
// With the power sampling set to 64, each loop takes 13.1 seconds, thus, 4.19mSec / iteration.
// With the power sampling set to 32, each loop takes 13.0 seconds, thus, 4.16mSec / iteration.
// With the power sampling set to 1, each loop takes 11.6 seconds, thus, 3.56mSec / iteration.
/*
		loop_counter--;

		if(!loop_counter)
			{
			beep(1843, 10);
			loop_counter = 3125;		// This should give a 10-second loop
			}
*/
		MAIN_TEST = !MAIN_TEST;			// Timing test J19-5 Check with oscilloscope for reversals

// Make atomic copy of KEY (TX request) signal for this processing round
		if(KEY)
			key = 1;					// Copy I/O bit to status flag	
		else
			key = 0;

// Auto band select
		auto_band_select();

// Measurements & basic calculations
		analog_measurements();

// Check alarm conditions
		check_alarms();					// Check if any alarms 

// Do front panel switches

// DISPLAY / FUNC
		if(DISP == 0)												// SW1
			{
			long_push = LONG_PUSH;									// Set long push timer

			if(lcd_mode == 0 && sub_page0 == MAX_SUB_PAGE0)
				beep(HZ466_85, cal.calval.beep_len);				// B-flat tone for basic display
			else
				{
				if(lcd_mode == 1 && sub_page1 == MAX_SUB_PAGE1)
					beep(HZ466_85, cal.calval.beep_len);			// B-flat tone for basic display
				else
					beep(HZ587_31, cal.calval.beep_len);			// D tone beep for other displays
				}

			while(DISP == 0)										// SW1
				{
				if(long_push == 0)
					{
					beep(HZ466_85, cal.calval.beep_len*10);			// Long beep for major page change
					break;
					}
				}
			if(long_push == 0)										// Major mode change
				{
				lcd_mode++;

				if(lcd_mode > MAX_LCD_MODE)
					lcd_mode = 0;

				while(DISP == 0);									// Wait until button released
				}
			else
				{
				if(lcd_mode == 0)									// Select sub page to adjust
					{
					sub_page0++;

					if(sub_page0 > MAX_SUB_PAGE0)
					sub_page0 = 0;
					}
				else
					{
					sub_page1++;

					if(sub_page1 > MAX_SUB_PAGE1)
					sub_page1 = 0;					
					}
				}
			encoder_get();											// Clear encoder after display change
			ms_delay(BUTTON_DEBOUNCE);
			}

// Operate
		if(!OPER)													// SW3
			{
			if(eeprom.defval.pa_state)
//			if(eeprom.defval.pa_state || alarms)					// If active alarms, always change to standby state
				{
				beep(HZ466_85, cal.calval.beep_len);				// B-flat tone beep for STANDBY
				eeprom.defval.pa_state = 0;
				}
			else
				{
				beep(HZ587_31, cal.calval.beep_len);				// D tone beep for OPER
				eeprom.defval.pa_state = 1;
				}

			while(OPER == 0);

			ms_delay(BUTTON_DEBOUNCE);
//			clear_alarms();											// Try to clear alarms
			}

// Auto

// 11.11.2008 toggle logic
		if(AUTO == 0)												// SW2
			{
			check_io_board_type();									// Check if F-sense board installed

			if(fsense_present != 0)									// Set auto only if F-sense board present
				{
				if(eeprom.defval.auto_band == 0)
					{
					eeprom.defval.auto_band = 1;					// Set automatic band select on
					reset_fsense();									// Reset F-sense logic
					band_query_timer = 0;							// Force band query if TRX2 protocol selected
					last_man_band = eeprom.defval.band;				// Save current selection
					eeprom.defval.band = NOT_KNOWN;
					beep(HZ587_31, cal.calval.beep_len);			// D tone beep
					}
				else
					{
					eeprom.defval.auto_band = 0;					// Set automatic band select off
					eeprom.defval.band = last_man_band;				// Restore last known manual band
					beep(HZ466_85, cal.calval.beep_len);			// B-flat tone beep
					}
				}
			else
				beep(HZ466_85, cal.calval.beep_len);				// B-flat tone beep, no chees

			ms_delay(BUTTON_DEBOUNCE);

			while(AUTO == 0);
			}

// Band UP & DOWN
		if(BAND_UP == 0)											// SW6 - Band+ button
			{
			if(eeprom.defval.auto_band != 0)
				{
				eeprom.defval.auto_band = 0;						// Set manual band select on
				eeprom.defval.band = last_man_band;					// Restore last known manual band
				beep(HZ466_85, cal.calval.beep_len);				// B-flat tone beep
				}
			else
				{
				eeprom.defval.band++;

				if(eeprom.defval.band > MAX_BAND)
					eeprom.defval.band = MAX_BAND;					// Stop at max

				if(eeprom.defval.band == MAX_BAND)
					beep(HZ698_45, cal.calval.beep_len);			// End-of-range F tone beep
				else
					beep(HZ587_31, cal.calval.beep_len);			// Normal D tone beep
				}

			ms_delay(BUTTON_DEBOUNCE);
			while(BAND_UP == 0);
			}

		if(BAND_DN == 0)											// SW4 - Band- button
			{
			if(eeprom.defval.auto_band != 0)
				{
				eeprom.defval.auto_band = 0;						// Set manual band select on
				eeprom.defval.band = last_man_band;					// Restore last known manual band
				beep(HZ466_85, cal.calval.beep_len);				// B-flat tone beep
				}
			else
				{
				eeprom.defval.band--;

				if(eeprom.defval.band < MIN_BAND)
					eeprom.defval.band = MIN_BAND;					// Stop at min

				if(eeprom.defval.band == MIN_BAND)
					beep(HZ466_85, cal.calval.beep_len);			// End-of-range B-flat tone beep
				else
					beep(HZ587_31, cal.calval.beep_len);			// Normal D tone beep
				}

			ms_delay(BUTTON_DEBOUNCE);
			while(BAND_DN == 0);
			}

//____________________________________________________________
// Do I/O

// Main board relays
			set_relays();								// Set gain & filter relays

// Evaluate TX possibility

			if(key == 1)								// TX request
				{
				if(eeprom.defval.pa_state == 1)			// OPER
					{	
					if((alarms == 0) && (eeprom.defval.band != NOT_KNOWN) && (eeprom.defval.band != OUT_OF_BAND))	// no alarms, band selected
						TX_ON = 1;						// RF on
					else
						TX_ON = 0;						// Something wrong, RF off
					}
				else
					TX_ON = 0;							// STANDBY, RF off
				}
			else
				{
				TX_ON = 0;								// No TX request RF off
				}

// Fan control
		fan_control();

//_____________________________________________________________
// Do LCD display

		set_cur_lcd(LINE1);								// Start from line 1

		switch(lcd_mode)								// Select major page to display
			{

// Normal display
			case 0: default:
// First draw graphical power meter
				display_pwr_meter();					// PWR meter bar graph

// Select meter to display
				switch(sub_page0)						// Select meter
					{
// Power
					case 0: default:					// Normal display
						lcdoutch(S_BAR);				// PWR-meter stopper bar
						disp_fwd_pwr();					// Display watt meter
					break;

// SWR
					case 1:								// Forward power
						lcdoutch(S_BAR);				// PWR-meter stopper bar

						if(swr == 0)					// No reliable reading
							lcd_putst("SWR N/A ");
						else
							disp_meter((double)swr, 0);
					break;

// Battery voltage
					case 2:								// Battery voltage mean value, measure only when display selected
						batt_mean_val += convert_adc12(BATT_CH);					// Measure & add
						bat_samples++;												// Add sample counter

						if(bat_samples > 15)										// 16 samples done ?
							{
							bat_samples = 0;
							ultemp = (batt_mean_val >> 4) * cal.calval.batt_mult;	// Scale Battery voltage
							batt_mean_val = 0;
							batt = ultemp / 1000;
							lcdoutch(S_BAR);										// PWR-meter stopper bar
							disp_meter((double)batt, 'V');							// V = batt / 1000
							}
					break;

// PA drain current
					case 3:										// Drain current, measure only when display selected
						lcdoutch(S_BAR);						// PWR-meter stopper bar
						disp_id();								// Measure & display drain current
					break;

// PA Temperature
					case 4:										// Display PA temperature
						lcdoutch(S_BAR);						// PWR-meter stopper bar
						disp_temp();
					break;
					}

// Second line
				set_cur_lcd(LINE2);								// Second display line

// Display gain setting
				switch(eeprom.defval.rf_gain[eeprom.defval.band])
					{	
					case 0:
						lcd_putst("G1");
					break;

					case 1:
						lcd_putst("G2");
					break;

					case 2:
						lcd_putst("G3");
					break;

					case 3:
						lcd_putst("G4");
					break;
					}

// Show PA state if no alarms
				if(alarms == 0)
					{
					if(eeprom.defval.pa_state == 0)
						lcd_putst(" STBY");

					if(eeprom.defval.pa_state == 1)
						{
						if(key)
							{
							if((alarms == 0) && (eeprom.defval.band != NOT_KNOWN) && (eeprom.defval.band != OUT_OF_BAND))	// no alarms, band selected
								lcd_putst(" TX  ");			// TX means PA on
							else
								lcd_putst(" NOP ");			// NOP means TX request but PA Off
							}
						else
							lcd_putst(" OPER");
						}
					}
				else	
					display_alarms();						// Display alarm message if active

// Show frequency select mode
				if(eeprom.defval.auto_band)
					lcd_putst(" A ");						// Auto mode
				else
					lcd_putst(" M ");						// Manual mode

// Show selected band
				if(eeprom.defval.Band_Units)					// Chech which units to display
					{
					lcd_putst("  ");
					lcd_putch(mb_txt[eeprom.defval.band][0]);	// Display selected band in metres
					lcd_putch(mb_txt[eeprom.defval.band][1]);
					lcd_putch(mb_txt[eeprom.defval.band][2]);

					if(eeprom.defval.band == 0 || eeprom.defval.band == 10)
						lcd_putch(' ');							// Display unit
					else
						lcd_putch('m');
					}
				else
					{
					lcd_putch(bs_txt[eeprom.defval.band][0]);	// Display selected band in MHz
					lcd_putch(bs_txt[eeprom.defval.band][1]);
					lcd_putch(bs_txt[eeprom.defval.band][2]);

					if(eeprom.defval.band == 0 || eeprom.defval.band == 10)
						lcd_putst("   ");						// Display unit
					else
						lcd_putst("MHz");
					}

// Adjust gain setting, buttons active only in normal display
				temp = encoder_get();

				if(temp > 0)
					{
					eeprom.defval.rf_gain[eeprom.defval.band]++;
					beep(HZ587_31, cal.calval.beep_len);	// D tone beep
					}

				if(temp < 0)
					{	
					eeprom.defval.rf_gain[eeprom.defval.band]--;
					beep(HZ587_31, cal.calval.beep_len);	// D tone beep
					}

				if(eeprom.defval.rf_gain[eeprom.defval.band] < 0)
					eeprom.defval.rf_gain[eeprom.defval.band] = 0;

				if(eeprom.defval.rf_gain[eeprom.defval.band] > 3)
					eeprom.defval.rf_gain[eeprom.defval.band] = 3;
				break;										// End of main display #0

//____________________________________________________________________
// User setup display page
			case 1:

// Select sub mode
			switch(sub_page1)								// Select sub page to display
					{

// Auto band select mode
					case 0:
						set_cur_lcd(LINE1);
						lcd_putst("Auto Band Detect");
						eeprom.defval.bsel_mode += encoder_get();							// Adjust band select mode

						if(eeprom.defval.bsel_mode < 0)										// Keep within limits
							eeprom.defval.bsel_mode = 0;

						if(eeprom.defval.bsel_mode > 2)
							eeprom.defval.bsel_mode = 2;

						set_cur_lcd(LINE2);
						lcd_putst("Select:");

						if(eeprom.defval.bsel_mode == 1)									// If band select set to TRX2 serial
							eeprom.defval.serial_mode = 0;									// Force serial mode to TRX2 protocol

						switch(eeprom.defval.bsel_mode)
							{
							case 0:
								lcd_putst("  F-Sense");
							break;

							case 1:
								lcd_putst("JUMA-TRX2");
							break;

							case 2:
								lcd_putst("   FT-817");
							break;
							}
					break;

// Serial I/O mode select
					case 1:
						set_cur_lcd(LINE1);
						lcd_putst("Serial Protocol ");

						if(eeprom.defval.bsel_mode == 1)										// If band select set to TRX2 serial
							eeprom.defval.serial_mode = 0;										// Force serial mode to TRX2 protocol
						else
						eeprom.defval.serial_mode += encoder_get();								// Adjust serial mode

						if(eeprom.defval.serial_mode < 0)										// Keep within limits
							eeprom.defval.serial_mode = 0;

						if(eeprom.defval.serial_mode > 1)
							eeprom.defval.serial_mode = 1;

						set_cur_lcd(LINE2);
						lcd_putst("RS-232:");

						switch(eeprom.defval.serial_mode)
							{
							case 0:
								lcd_putst("JUMA-TRX2");
							break;

							case 1:
								lcd_putst("Test Mode");
							break;
							}
					break;

// Select user UART1 baud rate
					case 2:
						set_cur_lcd(LINE1);
						lcd_putst("Serial Speed    ");
						eeprom.defval.br += encoder_get();													// Get encoder change

						if(eeprom.defval.br < 0)											// Keep within limits
							eeprom.defval.br = 0;

						if(eeprom.defval.br > 7)
							eeprom.defval.br = 7;

						SetUSART1baud(eeprom.defval.br);									// Set new Baud rate
						sprintf(lcdpbuff, "Baud Rate:%6s", br_txt[eeprom.defval.br]);		// Display current selection - Added size of string specifier - 5B4AIY
						set_cur_lcd(LINE2);
						lcd_putst(lcdpbuff);
					break;

// LCD back light				
					case 3: default:
						set_cur_lcd(LINE1);
						lcd_putst("LCD Backlighting");											// Alternatitive prompt - 5B4AIY
//					lcd_putst("Displ Brightness");											// Spelling correction - 5B4AIY
						eeprom.defval.back_light += encoder_get() * 10;						// Speed up the display adjustment to match the TRX-2 - 5B4AIY

						if(eeprom.defval.back_light > MAX_BL)
							eeprom.defval.back_light = MAX_BL;

						if(eeprom.defval.back_light < 0)
							eeprom.defval.back_light = 0;
						sprintf(lcdpbuff, "Level:%10i",eeprom.defval.back_light);				// Alternative display - 5B4AIY 
//						sprintf(lcdpbuff, " LCD BL = %4i ",eeprom.defval.back_light);			// Better formatting of O/P string - 5B4AIY
						set_cur_lcd(LINE2);
						lcd_putst(lcdpbuff);
						set_pwm3_dac(eeprom.defval.back_light);
					break;

// LCD Contrast
					case 4:
						set_cur_lcd(LINE1);
						lcd_putst("LCD Contrast    ");											// Alternative prompt - 5B4AIY
//						lcd_putst("Display Contrast");
						eeprom.defval.contrast += (encoder_get() * 50);

						if(eeprom.defval.contrast > MAX_CONTRAST)
							eeprom.defval.contrast = MAX_CONTRAST;

						if(eeprom.defval.contrast < 0)
							eeprom.defval.contrast = 0;
						sprintf(lcdpbuff, "Setting:%8i",eeprom.defval.contrast);			// Alternative display - 5B4AIY
//						sprintf(lcdpbuff, "Contrast = %4i  ",eeprom.defval.contrast);			// Better formatting of O/P string - 5B4AIY
						set_cur_lcd(LINE2);
						lcd_putst(lcdpbuff);
						set_pwm4_dac(eeprom.defval.contrast);
					break;

// SWR alarm limit				
					case 5:
						set_cur_lcd(LINE1);
						lcd_putst("SWR Trip Limit  ");							// Alternative prompr - 5B4AIY
						eeprom.defval.swr_limit += (encoder_get() * 10);

						if(eeprom.defval.swr_limit > MAX_SWR)
							eeprom.defval.swr_limit = MAX_SWR;

						if(eeprom.defval.swr_limit < 100)						// 1.00
							eeprom.defval.swr_limit = 100;

						sprintf(lcdpbuff, "Limit:%10.1f", ((double)eeprom.defval.swr_limit) / 100);	// Improved formatting - 5B4AIY
						set_cur_lcd(LINE2);
						lcd_putst(lcdpbuff);									// Improved formatting - 5B4AIY
					break;

// Select temperature units
					case 6:
						set_cur_lcd(LINE1);
						lcd_putst("Temperature: F/C");
						eeprom.defval.Temp_Units += encoder_get();							// Adjust serial mode

						if(eeprom.defval.Temp_Units < 0)									// Keep within limits
							eeprom.defval.Temp_Units = 0;

						if(eeprom.defval.Temp_Units > 1)
							eeprom.defval.Temp_Units = 1;

						set_cur_lcd(LINE2);
						lcd_putst("Units:");

						if(eeprom.defval.Temp_Units)
							lcd_putst("   Celsius");
						else
							lcd_putst("Fahrenheit");

						if(eeprom.defval.Temp_Units != temp_units)
							{
							temp_units = eeprom.defval.Temp_Units;

								if(eeprom.defval.Temp_Units)
									{
									eeprom.defval.temp_limit = TEMP_LIMITC;
									eeprom.defval.fan_start = FAN_STARTC;
									}
								else
									{
									eeprom.defval.temp_limit = TEMP_LIMITF;
									eeprom.defval.fan_start = FAN_STARTF;						
									}	
							}
					break;

// Fan control option
					case 7:
						set_cur_lcd(LINE1);
						lcd_putst("Fan Control     ");
						eeprom.defval.Fan_Control += encoder_get();
						
						if(eeprom.defval.Fan_Control > 3)
							eeprom.defval.Fan_Control = 3;

						if(eeprom.defval.Fan_Control < 0)
							eeprom.defval.Fan_Control = 0;

						set_cur_lcd(LINE2);

						switch(eeprom.defval.Fan_Control)
							{
							case 0:
								lcd_putst("Fan Speed:Normal");
							break;
							
							case 1:
								lcd_putst("Fan Speed:   Low");							
							break;
							
							case 2:
								lcd_putst("Fan Speed:Medium");
							break;
							
							case 3:
								lcd_putst("Fan Speed:  High");
							break;
							}
					break;

// Temperature alarm limit				
					case 8:
						set_cur_lcd(LINE1);
//						lcd_putst("Temp Alarm Limit");
						lcd_putst("Overtemp Limit  ");
						eeprom.defval.temp_limit += encoder_get();

						if(eeprom.defval.Temp_Units)
							{
							if(eeprom.defval.temp_limit > MAX_TEMPC)
								eeprom.defval.temp_limit = MAX_TEMPC;

							if(eeprom.defval.temp_limit < MIN_TEMPC)
								eeprom.defval.temp_limit = MIN_TEMPC;

							sprintf(lcdpbuff, "Limit:%8i\4C", eeprom.defval.temp_limit);	// Better formatting - 5B4AIY
							}
						else
							{
							if(eeprom.defval.temp_limit > MAX_TEMPF)
								eeprom.defval.temp_limit = MAX_TEMPF;

							if(eeprom.defval.temp_limit < MIN_TEMPF)
								eeprom.defval.temp_limit = MIN_TEMPF;

							sprintf(lcdpbuff, "Limit:%8i\4F", eeprom.defval.temp_limit);	// Better formatting - 5B4AIY
							}
						set_cur_lcd(LINE2);
						lcd_putst(lcdpbuff);
					break;

// Fan control				
					case 9:
						set_cur_lcd(LINE1);
						lcd_putst("Fan Cut-In Temp ");
						eeprom.defval.fan_start += encoder_get();
						
						if(eeprom.defval.Temp_Units)					// Celsius
							{
							if(eeprom.defval.fan_start < 0)
								eeprom.defval.fan_start = 0;

							if(eeprom.defval.fan_start > (eeprom.defval.temp_limit - 20))
								eeprom.defval.fan_start = eeprom.defval.temp_limit - 20;

							fan_stop = eeprom.defval.fan_start - 2;
							sprintf(lcdpbuff, "Start:%8i\4C", eeprom.defval.fan_start);	// Better formatting - 5B4AIY
							}
						else											// Fahrenheit
							{
							if(eeprom.defval.fan_start < 32)
								eeprom.defval.fan_start = 32;

							if(eeprom.defval.fan_start > (eeprom.defval.temp_limit - 40))
								eeprom.defval.fan_start = eeprom.defval.temp_limit - 40;

							fan_stop = eeprom.defval.fan_start - 4;
							sprintf(lcdpbuff, "Start:%8i\4F", eeprom.defval.fan_start);	// Better formatting - 5B4AIY
														}
						set_cur_lcd(LINE2);
						lcd_putst(lcdpbuff);
					break;

					case 10:
						set_cur_lcd(LINE1);
						lcd_putst("Band Units MHz/m");
						eeprom.defval.Band_Units += encoder_get();

						if(eeprom.defval.Band_Units < 0)								// Keep within limits
							eeprom.defval.Band_Units = 0;

						if(eeprom.defval.Band_Units > 1)
							eeprom.defval.Band_Units = 1;

						set_cur_lcd(LINE2);
						lcd_putst("Units:    ");

						if(eeprom.defval.Band_Units)
							lcd_putst("Metres");
						else
							lcd_putst("   MHz");
					break;
					}					// End switch sub_page1
				break;					// End of main disp #1
			}							// Switch(lcd_mode)  end

// Run serial interface functions

		switch(eeprom.defval.serial_mode)
			{
			case 0:						// JUMA-TRX2 command set
				serial_pa100();
			break;

			case 1: default:
				serial_test();			// Test commands
			break;
			}

// Power switch

		if(PWR_SW)						// SW7
			power_off();				// Go shut down

		}	// End for(;;)
	}		// End main

