// JUMA-PA100 protocol
// Juha Niinikoski, OH2NLT 06.07.2006

// Active only if TRX2 protocol is selected 15.11.2008

#include "juma-pa100.h"				// Board definitions

// Global variables

int poll_resp_rec;					// Poll response (freq set) received from TRX2, 1 = rec

// Module variables

#define CMD_BUF_SIZE 8

static char cmd_buf[CMD_BUF_SIZE];	// Command buffer
static int cmd_buf_idx;				// Buffer fill pointer

// External references
// External functions

extern void putch(char);			// UART I/O
extern unsigned char kbhit(void);
extern unsigned char getch(void);
extern unsigned char getche(void);
extern unsigned long getlong(void);
extern void bin2ascii(unsigned long);
extern void putlong( unsigned long); 

// EEPROM structures

#include "pa100_eeprom.h"			// Get EEPROM structure definitions

extern union
	{
	struct defval defval;
	unsigned int storage[sizeof(struct defval) / 2];
	}eeprom;

// System calibration values

extern union
	{
	struct calval calval;
	unsigned int ee[sizeof(struct calval) / 2];
	}cal;


// TRX2 protocol parser

// Send band data
void send_band_data(void)
	{
	putch('=');
	putch('B');

	if(eeprom.defval.band == NOT_KNOWN)
		putch('0');											// 0 = Unknown
	else
		putch((char)(eeprom.defval.band & 0x0F) + 0x30);	// Send band digit

	putch(0x0A);				// LF
	putch(0x0D);				// CR
	}

// Query band data
void query_band_data(void)
	{
	putch('?');
	putch('B');
	putch(0x0A);				// LF
	putch(0x0D);				// CR
	}

// Clear buffer
void clear_buffer(void)
	{
	int x;

	for(x = 0; x < CMD_BUF_SIZE; x++)
		{
		cmd_buf[x] = 0;
		}
	cmd_buf_idx = 0;
	}

// Filter no operation characters, return = 1 if no op
int filter_nop_char(char c)
	{
	if((c==0x00) || (c==0x0A) || (c==0x20) || (c==0x2C) || (c==0x2E))
		return(1);
	else
		return(0);
	}

// Parse TRX2 message
void serial_pa100(void)
	{
	int hit, b;
	char c;

	hit = kbhit();
	if(hit)
		{
		c = getch();								// Save character

//		if(hit != 0)												// test
//			printf("\n\rhit %i, %X, %i", hit, c, cmd_buf_idx);		// test

// filter nop
		if(filter_nop_char(c))						// Discard nop characters 
			return;									// Do nothing

// Capture start sync
	if(c == '=' || c == '?')						// Sync with action characters
		clear_buffer();

// Check if buffer ready, interpret message
		if(c == 0x0D)								// CR = ready
			{
			switch(cmd_buf[0])						// Action
				{
				case '?':							// Query
					if(cmd_buf[1] == 'B')			// Band Data
						{
						send_band_data();			// Send answer
						}
					clear_buffer();					// Clear for next round
				break;

				case '=':							// Set
					if(cmd_buf[1] == 'B')			// Band Data
						{
						b = cmd_buf[2] & 0x0F;		// Brute force ASCII to binary

						if(b < 10 && eeprom.defval.bsel_mode == 1 && eeprom.defval.auto_band == 1)	// Band in range and TRX2 protocol and auto
							{
							eeprom.defval.band = b;	// Set band data
							poll_resp_rec = 1;		// Flag for poll response timeout	
							}
						}
					clear_buffer();					// Clear for next round
				break;

				default:							// Unknown
					clear_buffer();					// Clear for next round
				break;
				}
			}

		else										// Collect data
			{
			cmd_buf[cmd_buf_idx] = c;				// Save data
			if(cmd_buf_idx < CMD_BUF_SIZE-1)
				cmd_buf_idx++;						// Adjust index
			}
		}
	}

