// JUMA-PA100 serial I/O test command set
// Juha Niinikoski, OH2NLT 06.07.2006

#include "juma-pa100.h"				// Board definitions
#include "DataEEPROM.h"				// EEPROM I/O definitions, interface to assembler module
#include "lcd-trx2.h"				// LCD specific definitions
#include <stdio.h>					// Added - 5B4AIY

// External references
// External functions

extern void putch(char);						// UART I/O
extern unsigned char kbhit(void);
extern unsigned char getch(void);
extern unsigned char getche(void);
extern unsigned long getlong(void);
extern unsigned char get2hex(void);
//extern void bin2ascii(unsigned long);
//extern void putlong( unsigned long); 

extern void draw_s_meter(int);					// LCD functions
//extern void draw_bar(int, int);
//extern void lcd_putst(register const char *);
//extern void set_chgen(void);
//extern void initlcd(void);
extern void clear_lcd(void);
//extern void set_cur_lcd(unsigned char);
//extern void disp_freq(long);
//extern void disp_rit(int);
//extern void disp_meter(double, char);
extern void lcd_putch(unsigned char);			// Added - 5B4AIY

//extern void init_adc12(void);
extern int convert_adc12(unsigned int adchs);	// ADC

//extern void init_spi1(void);					// SPI, main board 74HC595 I/O latch
extern void exchg_data_spi1(unsigned int);


extern void ms_delay( unsigned int);			// General delay

extern void beep(int, int);						// Added - 5B4AIY
extern int eval_y817_band(void);				// Get Yaesu 817 band data

// External data

extern const char ver[];			// System version

//extern int batt;					// Power supply voltage scaled value, measured only when displayed
//extern int service_mode;			// Service mode selected flag

//extern unsigned int long_push;		// Long push timer for buttons

//extern unsigned int rf_attn;		// PA RF attenuator
//extern int rf_filt_idx;			// RF filter & attenuator switch table index
//extern unsigned int hc595;			// Main & PA board SPI bus bits

//extern int PTT_IN;					// Atomic copy of PTT_IN_IO input / TX on, 0 = active

extern int fsense_tst;				// Flag for F-sense test printouts
extern int fcy;
extern char br_txt[8][8];
extern int alarms;

// EEPROM structures

#include "pa100_eeprom.h"			// Get EEPROM structure definitions

extern union
	{
	struct defval defval;
	unsigned int storage[sizeof(struct defval) / 2];
	}eeprom;

// System calibration values

extern union
	{
	struct calval calval;
	unsigned int ee[sizeof(struct calval) / 2];
	}cal;

void dump_eeprom(void)
	{
	int x;
	unsigned int y, w;

	printf("\n\rDump EEPROM contents\n\r");	
	for(x = 0; x < 256; x = x + 2)						// Byte address for EEPROM, but data is stored in words
		{
		y = ReadEE(EEPAGE, (x + EEDEF), &w, WORD);		// EEPROM Address 8 high bits, address + physical EEPROM start 16 low bits
		if((x % 16) == 0)
			printf("\n\r%5.4X", x);

		printf("%5.4X", w);			
		}
	printf("\n\rLast success code = %X\n\r", y);
	}

void serial_test(void)
	{
	int x, z, temp;
	double v;
	long f;
//	unsigned int y, w;				// w unused - 5B4AIY
	unsigned int y;					// Added - 5B4AIY 
	unsigned char c;

	if(kbhit())
		{
		c = getch();
		switch(c)
			{

// Info commands
			case 'I': case '?': case 'H': case'h':					// System info
				printf("\n\rJUMA-TRX2 Copyright OH7SV & OH2NLT\n\r");
				printf("\n\rSoftware version: ");
				printf(ver);
				printf(", Copyright Juha Niinikoski OH2NLT\n\r");
				printf("Additional features and modifications - Adrian Ryan 5B4AIY\n\r");
				printf("\n~~~~~~~~~~~~~~[COMMAND TABLE]~~~~~~~~~~~~~~\n\r");
				printf("I\tHelp - (This Screen)\n\r");
				printf("A\tADC Channel Dump\n\r");
				printf("a\tTest Alarm System\n\r");
				printf("b\tLCD Bar Graph Test, ESC to exit\n\r");
				printf("C\tClear Factory Default reset counter\n\r");
				printf("D\tDump System & User Settings\n\r");
				printf("E\tDump EEPROM contents\n\r");
				printf("f\tToggle Frequency Sense On/Off\n\r");
				printf("t\tBuzzer Sound Test\n\r");
				printf("W\tWrite ASCII to LCD, ESC to exit\n\r");
				printf("w\tWrite HEX to LCD, 1B to exit\n\r");
				printf("c\tContinuous SPI write\n\r");
				printf("Z\tDivide-By-Zero Trap\n\n\r");
			break;

// Test commands

// ADC & PWM DAC test
			case 'A':												// Measure all ADC channels 9:14
			for(x = 9; x < 15; x++)
				{
				temp = convert_adc12(x);
				v = (double)temp;
				v *= AD_SCALE_FACTOR;
				printf("\n\rADC %2i = %4i Scaled: %5.3fV ", x, temp, v / 1000000.0);

				switch(x)
					{
					case 9:
						v = ((double)temp * cal.calval.id_mult);
						printf("%7.3f Amps", v / 200000.0);
					break;

					case 10:
						v = (double)temp;
						v *= cal.calval.batt_mult;
						printf("%7.3f Volts", v / 1000000.0);
					break;

					case 11:
						z = eval_y817_band();
						switch(z)
							{
							case 0:
								printf("  %i Out-Of-Band", z);
							break;

							case 1:
								printf("  %i 1.8MHz Filter", z);
							break;

							case 2:
								printf("  %i 3.5MHz Filter", z);
							break;

							case 3:
								printf("  %i 7.0MHz Filter", z);
							break;

							case 4:
								printf("  %i 10MHz Filter", z);
							break;

							case 5: case 6:
								printf("  %i 14MHz - 18MHz Filter", z);
							break;

							case 7: case 8: case 9:
								printf("  %i 21MHz - 28MHz Filter", z);
							break;

							case 10:
								printf("  %i Unknown Band", z);
							break;
							}
					break;

					case 12:
						v = (double)temp;
						v *= v;
						v *= cal.calval.rev_pwr_mult;
						printf("%5.1f Watts (Reverse)", v / 100000000.0);
					break;

					case 13:
						v = (double)temp;
						v *= v;
						v *= cal.calval.fwd_pwr_mult;
						printf("%5.1f Watts (Forward)", v / 100000000.0);
					break;

					case 14:
						if(eeprom.defval.Temp_Units)
							{
							v = (double)temp;
							v = (v * T_MULTC) / 65536.0;
							printf("%5.0f Celsius", v);
							}
						else
							{
							v = (double)temp;
							v = ((v * T_MULTF) / 65536.0) + 32.0;
							printf("%5.0f Fahrenheit", v);
							}					
					break;
					}
				}
				printf("\n\r");
			break;

			case'a':
				printf("Enter 1, 2, 3, 4, 5, or 6 ...\n\r");

				while(!kbhit());	//	while(kbhit() == 0);

				c = getch();

				switch(c)
					{
					case '1':
						printf("1 - High SWR\n\r");
						alarms = alarms | SWR_AL;
					break;

					case '2':
						printf("2 - Over-Current\n\r");
						alarms = alarms | CURR_AL;
					break;

					case '3':
						printf("3 - Over-Temperature\n\r");
						alarms = alarms | TEMP_AL;
					break;

					case '4':
						printf("4 - High Voltage\n\r");
						alarms = alarms | HI_V;
					break;

					case '5':
						printf("5 - Low Battery Pre-Limit\n\r");
						alarms = alarms | LO_BATT;
					break;

					case '6':
						printf("6 - Low Battery Limit\n\r");
						alarms = alarms | LO_V;
					break;
					}
			break;

// Dump System & User Settings
			case'D':
				printf("\n\rSystem Calibration Settings\n\r");
				printf("Battery Voltage Factor : %li\n\r", cal.calval.batt_mult);
				printf("PA Current Factor      : %li\n\r", cal.calval.id_mult);
				printf("Power Meter Factor     : %li\n\r", cal.calval.fwd_pwr_mult);
				printf("Beep Length Time       : ");

					if(cal.calval.beep_len)
						printf("%i mS\n\r", cal.calval.beep_len);
					else
						printf("OFF\n\r");
				printf("Serial Link Polling    : ");

					if(cal.calval.q_timer)
						printf("%.2f Seconds\n\r", (double)(cal.calval.q_timer) / 1000);
					else
						printf("Off\n\r");
				printf("PWR Measurement Samples: %i\n\r", cal.calval.samples);
				printf("Over-Voltage Trip      : ");

				if(cal.calval.alarm_flags & 0x01)
					{
					f = (long)cal.calval.overvoltage_trip * (long)cal.calval.batt_mult;
					v = ((double)f) / 1000000.0;
					printf("%4i = %5.2fV\n\r", cal.calval.overvoltage_trip, v);
					}
				else
					printf("OFF\n\r");

				printf("Under-Voltage Trip     : ");

				if(cal.calval.alarm_flags & 0x02)
					{
					f = (long)cal.calval.undervoltage_trip * (long)cal.calval.batt_mult;
					v = ((double)f) / 1000000.0;
					printf("%4i = %5.2fV\n\r", cal.calval.undervoltage_trip, v);
					}
				else
					printf("OFF\n\r");

				printf("\n\rUser Configuration Settings\n\r");
				printf("LCD Contrast           : %i\n\r", eeprom.defval.contrast);
				printf("LCD Backlighting       : %i\n\r", eeprom.defval.back_light);
				printf("RS-232 Port Mode       : Test Mode\n\r");
				printf("RS-232 Port Speed      : %s Baud\n\r", br_txt[eeprom.defval.br]);
				printf("Frequency Sense Mode   : ");
					switch(eeprom.defval.bsel_mode)
						{
						case 0:
							printf("Frequency Sense\n\r");
						break;
						
						case 1:
							printf("JUMA TRX2\n\r");
						break;
						
						case 2:
							printf("Yaesu FT-817\n\r");
						break;
						}
				printf("SWR Trip Limit         : %.2f\n\r", (double)(eeprom.defval.swr_limit) / 100);
				printf("Temperature Units      : ");

					if(eeprom.defval.Temp_Units)
						printf("Celsius\n\r");
					else
						printf("Fahrenheit\n\r");
				printf("Fan Speed Control      : ");
					switch(eeprom.defval.Fan_Control)
						{
						case 0:
							printf("Normal\n\r");
						break;
						
						case 1:
							printf("Low Speed ON\n\r");
						break;
						
						case 2:
							printf("Medium Speed ON\n\r");
						break;
						
						case 3:
							printf("High Speed ON\n\r");
						break;
						}
				printf("Temperature Alarm Limit: ");

					if(eeprom.defval.Temp_Units)
						printf("%iC\n\r", eeprom.defval.temp_limit);
					else
						printf("%iF\n\r", eeprom.defval.temp_limit);
				printf("Fan Start Temperature  : ");

					if(eeprom.defval.Temp_Units)
						printf("%iC\n\r", eeprom.defval.fan_start);
					else
						printf("%iF\n\r", eeprom.defval.fan_start);
				printf("Current Amplifier State: ");

					if(eeprom.defval.pa_state)
						printf("Operate\n\r");
					else
						printf("Standby\n\r");
				printf("Band Select            : ");

					if(eeprom.defval.auto_band)
						printf("Auto\n\r");
					else
						printf("Manual\n\r");
				printf("Band Display Units     : ");

					if(eeprom.defval.Band_Units)
						printf("Metres\n\r");
					else
						printf("MHz\n\r");
			break;

// EEPROM
			case 'E':												// Dump EEPROM
				dump_eeprom();
			break;

			case 'C':												// Clear factory default reset counter
				printf("\n\rEEPROM reset counter cleared");
				temp = 0;
				EraseEE(EEPAGE, EE_FD_LOC, WORD);
				WriteEE(&temp, EEPAGE, EE_FD_LOC, WORD);
				dump_eeprom();
			break;

// LCD tests
			case 'W':												// Test LCD character generator, ASCII
				printf("Write ASCII to LCD\n\r");
				clear_lcd();

				while(c != 0x1B)
					{
					while(!kbhit());
//					while(kbhit() == 0);
					c = getch();

					if(c == 0x0D)
						clear_lcd();								// CR = clear screen

					printf("Char = %X\n\r", c);
					lcd_putch(c);
					}
			break;

			case 'w':												// Test LCD character generator, HEX
				printf("write HEX to LCD\n\r");
				clear_lcd();

				while(c != 0x1B)
					{
					c = get2hex();

					if(c == 0x0D)
						clear_lcd();								// CR = clear screen

					printf(" = %c\n\r", c);
					lcd_putch(c);
					}
			break;

			case 'b':												// Bar graph
				printf("Bar Graph Test\n\r");
					
				for(;;)
					{
					for(x = 0; x < 96; x++)
						{
						clear_lcd();
						draw_s_meter(x);
						ms_delay(50);
						}

					if(kbhit())
						break;
					}
			break;

// Temporary hardware debug commands

			case 't':						// Buzzer sound test, pitch = FCY MHz / tone(Hz) * 2
				printf("\n\rSound Test\n\rEnter Frequency (100 - 5000Hz): ");
				f = getlong();

				if(f < 100 || f > 5000)
					{
					printf("Illegal Value!\n\r");
					break;
					}

				f = FCY / (f << 1);
				printf("Data Value: %li\n\r", f);
				printf("Enter duration (mS): ");
				z = getlong();
				beep(f, z);
			break;

			case 'c':						// Continous test write SPI bus
				printf("\n\rSPI Tx continuous, any key to exit...");
				for(;;)
					{
					y++;
					exchg_data_spi1(y);
					ms_delay(10);

					if(kbhit())
						break;
					}
			break;

			case 'f':						// F-sense test On / Off
				if( fsense_tst == 0)
					{
					fsense_tst = 1;
					printf("\n\rF-Sense test: ON");
					}
				else
					{
					fsense_tst = 0;
					printf("\n\rF-Sense test: OFF");
					}
			break;

			case 'Z':						// Trap / Error test
				printf("\n\rDiv by Zero");
				x = 0;
				temp = temp / x;			// Generate error ==> trap
			break;

			default:
				printf("Key is: %d\n\r", c);
				printf("Try: Valid commands\n\r");
				}	// case
			}		// if
	}

