// JUMA-TRX2 SPI 1 driver for 74HC595 ahift registers
// Two 74HC595 can be connected in cascade
// Juha Niinikoski, OH2NLT 03.12.2006
 
// MPLAB C30 version

// MCU dsPIC30F6014A
// Quiet bus test to reduce main board interference 23.12.2006

#include "juma-pa100.h"				// JUMA-TRX2 hardware specific definitions

#define SPI1_LATCH _LATB2			// Strobe signal in I/O pins

unsigned int spi_rx_data;			// RX data
int spi_rx_data_available;			// Data available flag

static unsigned int old_dta;		// Previously transmitted data.

// Init SPI 1 system
void init_spi1(void)
	{
	SPI1_LATCH = 0;					// Set Slave Latch to idle state, lov
	spi_rx_data_available = 0;
	old_dta = 0;
	SPI1CON = 0x0439;				// Bit clock falling edge, clock/32
	SPI1STAT = 0x8000;				// SPI1 on
	_SPI1IF = 0;
	_SPI1IE = 1;					// Enable SPI TX ready IRQ
	}

// Send data
void exchg_data_spi1(unsigned int dta)
	{
	if(dta != old_dta)				// Transmit only if data changed
		{
    	while(SPI1STATbits.SPITBF);	// Wait until 'SPITBF' bit is cleared
		SPI1BUF = dta;
		old_dta = dta;
		}
	}

// TX ready, generate latch pulse

void __attribute__((interrupt)) _SPI1Interrupt(void)
	{
//	int x;							// Unused - 5B4AIY
	SPI1_LATCH = 1;					// Generate latch strobe pulse __-__, rising edge sensitive
	_SPI1IF = 0;					// Clear IRQ flag

	if(SPI1STATbits.SPIRBF)
		{
		spi_rx_data = SPI1BUF;		// Save RX data
		spi_rx_data_available = 1;	// Data available
		}

	SPI1_LATCH = 0;					// Back to idle state
	}

